#!/usr/bin/perl -w

=head1 NAME

B<POWER::SSL> - 

=head1 SYNOPSIS

 use POWER::SSL;



=head1 DESCRIPTION

 :

 SSL_NULL_WITH_NULL_NULL
 SSL_RSA_EXPORT_WITH_RC4_40_MD5
 SSL_RSA_WITH_RC4_128_MD5

=head1 FUNCTIONS

=over 4

=cut

package POWER::SSL;

use Errno qw( EINPROGRESS EAGAIN );
use Digest::MD5 qw(md5);
use Digest::SHA1 qw(sha1);
use Crypt::RC4;

use strict;
use vars qw($VERSION); # TODO Public Vars list

$VERSION = "0.02";

sub import {
    no strict;
    *{"${\scalar caller}::$_"} = \&{$_} for qw(); # TODO Public list
}			

# TODO <PublicVar>;

##############################################################################
###############   POWER::RSA   ###############################################
##############################################################################

use Math::Pari qw(PARI Mod lift floor pari2num);

sub i2osp {
    my $num = PARI(shift); 
    my $d = $num;
    my $l = shift || 0;
    my $base = PARI(256); my $result = '';
    if ($l) { return if $num > $base ** $l }

    do { 
        my $r = $d % $base;
        $d = ($d-$r) / $base;
        $result = chr($r) . $result;
    } until ($d <= $base);
    $result = chr($d) . $result if $d != 0;

    if (length($result) < $l) { 
        $result = chr(0)x($l-length($result)) . $result;
    }
    return $result;
}

sub os2ip {
    my $string = shift;
    my $base = PARI(256);
    my $result = 0;
    my $l = length($string); 
    for (0 .. $l-1) {
        my ($c) = unpack "x$_ a", $string;
        my $a = int(ord($c));
        my $val = int($l-$_-1); 
        $result += $a * ($base**$val);
    }
    return $result;
}

sub bitsize ($) {
    return pari2num(floor(Math::Pari::log(shift)/Math::Pari::log(2)) + 1);
}

sub octet_len { 
    return pari2num(floor((PARI(bitsize($_[0]))+7)/8));
}

sub PKCS1v15_block1 {
    my ($message, $modulus) = @_;
    my $len = octet_len($modulus) - 1;
    die "PKCS1v15_block1: modulus too short" if $len < length($message) + 10;
    return "\x00\x01" . ("\xff" x ($len-length($message)-2)) . "\x00" . $message;
}

sub PKCS1v15_block2 {
    my ($message, $modulus) = @_;
    my $len = octet_len($modulus) - 1;
    die "PKCS1v15_block2: message too long" if length($message) > $len - 10;
    my $rndx = "";
    open my $RND, "/dev/urandom" or die "open /dev/urandom: $!";
    while (length($rndx) < $len-length($message)-2) {
        die "can't read /dev/urandom" if 1 != sysread($RND, my $rnd1, 1);
	$rndx .= $rnd1 if $rnd1 ne "\x00";
    }
    # FIXME TRACE
    $rndx = __trace2bin("
	e9 e9 d9 22 16 c1 1c 4f 1d 69 e8 04 ae
	");
    close $RND;
    return "\x00\x02" . $rndx . "\x00" . $message;
}

sub encryptblock {
    my ($modulus) = @_;
    return octet_len($modulus) - 11;
}

sub decryptblock {
    my ($modulus) = @_;
    return octet_len($modulus);
}

sub blocksize {
    my ($blocksize, $message) = @_;
    return length($message) if $blocksize == -1;
    return 0 if $blocksize < 1;
    return $blocksize;
}

sub steak {
    my ($text, $blocksize) = @_;
    my $textsize = length($text);
    my $chunkcount = $textsize % $blocksize
        ? int($textsize/$blocksize) + 1 : $textsize/$blocksize;
    my @segments = unpack "a$blocksize"x$chunkcount, $text;
    return @segments;
}

sub RSA { lift Mod($_[0], $_[1]) ** $_[2] }

##############################################################################
##############################################################################
##############################################################################

# Used to pack packets, in "client_hello" and "server_hello".
my %Protocol = (	    #	C
    major		    => 3,   # 0x03
    minor		    => 0,   # 0x00
    );
# Used in "client_hello" and "server_hello"
my %CipherSuite = (		    #	CC
    SSL_NULL_WITH_NULL_NULL	    => "\x00\x00",
    SSL_RSA_EXPORT_WITH_RC4_40_MD5  => "\x00\x03",
    SSL_RSA_WITH_RC4_128_MD5	    => "\x00\x04",
    );
my %CipherSpec = (
    SSL_NULL_WITH_NULL_NULL         => {
	bulk_cipher_algorithm   => "null",  # null, rc4, rc2, des, 3des, des40, fortezza
	mac_algorithm		=> "null",  # null, md5, sha
	cipher_type		=> "stream",# stream, block
	is_exportable		=> "true",  # true, false
	hash_size		=> 0,	    # C
	key_material		=> 0,	    # C
	IV_size			=> 0,	    # C
	IsEx			=> 1,
	KeyExchange		=> "null",
	},
    SSL_RSA_EXPORT_WITH_RC4_40_MD5  => {
	bulk_cipher_algorithm   => "rc4",   # null, rc4, rc2, des, 3des, des40, fortezza
	mac_algorithm		=> "md5",   # null, md5, sha
	cipher_type		=> "stream",# stream, block
	is_exportable		=> "true",  # true, false
	hash_size		=> 16,	    # C
	key_material		=> 5,	    # C
	IV_size			=> 0,	    # C
	IsEx			=> 1,
	KeyExchange		=> "rsa",
	},
    SSL_RSA_WITH_RC4_128_MD5        => {
	bulk_cipher_algorithm   => "rc4",   # null, rc4, rc2, des, 3des, des40, fortezza
	mac_algorithm		=> "md5",   # null, md5, sha
	cipher_type		=> "stream",# stream, block
	is_exportable		=> "false", # true, false
	hash_size		=> 16,	    # C
	key_material		=> 16,	    # C
	IV_size			=> 0,	    # C
	IsEx			=> 0,
	KeyExchange		=> "rsa",
	},
    );
# Used in "client_hello" and "server_hello"
my %CompressionMethod = (   #	C
    null		    => 0,   # 0x00
    );
# Used to pack/unpack packets
my %HandshakeType = (	    #	C
    hello_request	    => 0,   # 0x00
    client_hello	    => 1,   # 0x01
    server_hello	    => 2,   # 0x02
    certificate		    => 11,  # 0x0B
    server_key_exchange	    => 12,  # 0x0C
    certificate_request	    => 13,  # 0x0D
    server_hello_done	    => 14,  # 0x0E
    certificate_verify	    => 15,  # 0x0F
    client_key_exchange	    => 16,  # 0x10
    finished		    => 20,  # 0x14
    );
my %ChangeCipherSpec = (    #	C
    change_cipher_spec	    => 1,   # 0x01
    );
my %AlertLevel = (	    #	C
    warning		    => 1,   # 0x01
    fatal		    => 2,   # 0x02
    );
my %AlertDescription = (    #	C
    close_notify	    => 0,   # 0x00
    unexpected_message	    => 10,  # 0x0A
    bad_record_mac	    => 20,  # 0x14
    decompression_failure   => 30,  # 0x1E
    handshake_failure	    => 40,  # 0x28
    no_certificate	    => 41,  # 0x29
    bad_certificate	    => 42,  # 0x2A
    unsupported_certificate => 43,  # 0x2B
    certificate_revoked	    => 44,  # 0x2C
    certificate_expired	    => 45,  # 0x2D
    certificate_unknown	    => 46,  # 0x2E
    illegal_parameter 	    => 47,  # 0x2F
    );
my %Finished = (
    client		    => "\x43\x4C\x4E\x54",
    server		    => "\x53\x52\x56\x52",
    );
# Used to pack/unpack packets
my %ContentType = (	    #	C
    change_cipher_spec	    => 20,  # 0x14
    alert		    => 21,  # 0x15
    handshake		    => 22,  # 0x16
    application_data	    => 23,  # 0x17
    );
# Used to pack packets
my $MaxFragmentLength	    = 2**14;
# Used in "certificate_verify" and "finished"
my %pad1 = (
    md5			    => chr(0x36) x 48,
    sha			    => chr(0x36) x 40,
    );
my %pad2 = (
    md5			    => chr(0x5C) x 48,
    sha			    => chr(0x5C) x 40,
    );
%$_ = (reverse(%$_), %$_) for \%ContentType, \%AlertLevel, \%AlertDescription,
    \%HandshakeType, \%CipherSuite, \%CompressionMethod;
# Client capabilities
my $Protocol = pack("CC", @Protocol{"major","minor"});
my $CipherSuites = join "", @CipherSuite{'SSL_NULL_WITH_NULL_NULL',
    'SSL_RSA_EXPORT_WITH_RC4_40_MD5', 'SSL_RSA_WITH_RC4_128_MD5'};
my $CompressionMethods = pack("C*", $CompressionMethod{null});

##############################################################################
##############################################################################
##############################################################################

sub _get_clean_session {
  return {
    # An arbitrary byte sequence chosen by the server to identify an active
    # or resumable session state
    ID			    => "",	# <0..32>
    # X509.v3[X509] certificate of the peer. This element of the state may
    # be null.
    PeerCert		    => "",	# <2**24-1> X509.v3 / ASN.1 <0..2**24-1>
    # A flag indicating whether the session can be used to initiate new
    # connections.
    IsResumable		    => "false",	# true, false
    # The algorithm used to compress data prior to encryption. 
    Compression		    => $CompressionMethod{null},    # FIXME "null" !?
    # Specifies the bulk data encryption algorithm (such as null, DES, etc.)
    # and a MAC algorithm (such as MD5 or SHA). It also defines cryptographic
    # attributes such as the hash_size. (See Appendix A.7 for formal
    # definition.)
    CipherSpec		    => {
	bulk_cipher_algorithm   => "null",  # null, rc4, rc2, des, 3des, des40, fortezza
	mac_algorithm		=> "null",  # null, md5, sha
	cipher_type		=> "stream",# stream, block
	is_exportable		=> "true",  # true, false
	hash_size		=> 0,	    # C
	key_material		=> 0,	    # C
	IV_size			=> 0,	    # C
	IsEx			=> 1,
	KeyExchange		=> "null",
	},
    # 48-byte secret shared between the client and server. 
    MasterSecret	    => "",	# <48>
    Connection		    => {
	# Byte sequences that are chosen by the server and client for each
	# connection. 
	ServerRandom		=> "",
	ClientRandom		=> "",
	# The secret used in MAC operations on data written by the server. 
	ServerWriteMACSecret    => "",
	# The secret used in MAC operations on data written by the client. 
	ClientWriteMACSecret    => "",
	# The bulk cipher key for data encrypted by the server and decrypted by
	# the client. 
	ServerWriteKey		=> { m => undef, e => undef },
	# The bulk cipher key for data encrypted by the client and decrypted by
	# the server. 
	ClientWriteKey		=> "",
	# When a block cipher in CBC mode is used, an initialization vector
	# (IV) is maintained for each key. This field is first initialized by
	# the SSL handshake protocol. Thereafter the final ciphertext block
	# from each record is preserved for use with the following record.
	ServerIV		=> "",
	ClientIV		=> "",
	# Each party maintains separate sequence numbers for transmitted and
	# received messages for each connection. When a party sends or receives
	# a change cipher spec message, the appropriate sequence number is set
	# to zero. Sequence numbers are of type uint64 and may not exceed
	# 2^64-1.
	ServerSEQ		=> 0,	    # N,    64 bit   FIXME
	ClientSEQ		=> 0,	    # N,    64 bit   FIXME
	},
    # Temporary, used to generate MasterSecret
    PreMasterSecret	    => "",	# <48>
    # Used in "finished" packet
    HandshakeMessages	    => "",
  }
}
    
sub _ClientHello {
    my $self = shift;
    open my $RND, "/dev/urandom" or die "open /dev/urandom: $!";
    die "can't read /dev/urandom" if 28 != sysread($RND, my $rnd28, 28);
    close $RND;
    $self->{Pending}{Connection}{ClientRandom} = pack("N", CORE::time()).$rnd28;
    # FIXME TRACE
#    $self->_pack_handshake("client_hello", $Protocol .
#	$self->{Pending}{Connection}{ClientRandom} .
#	pack("C/a* n/a* C/a*", $self->{Pending}{ID}, $CipherSuites,
#	$CompressionMethods));
    my $CipherSuites = __trace2bin("
	01 00 80 03 00 80 06 00 40 07 00 c0 00 00 04 00
	00 0a 00 00 09 00 00 03 00 00 06
	");
    my $TRACE_sessionid = "";
    my $TRACE_challenge = __trace2bin("
	d8 90 d7 86 4e 5c 92 9f 90 07 da 83 1c 25 89 01
	");
    $self->{Pending}{Connection}{ClientRandom} = ("\x00" x 16).$TRACE_challenge;
    my $SSLv2handshake = pack("C", $HandshakeType{client_hello}) . $Protocol .
	pack("nnn", length($CipherSuites), length($TRACE_sessionid),
	length($TRACE_challenge)) . $CipherSuites . $TRACE_sessionid .
	$TRACE_challenge;
    $self->{Pending}{HandshakeMessages} .= $SSLv2handshake;
    my $SSLv2 = "\x80" . pack("C", length($SSLv2handshake)) . $SSLv2handshake;
    $self->{writebuf} .= $SSLv2;
    warn " >   SSLv2 client_hello\n --> handshake\n".__DUMP($SSLv2);
}

sub _ServerHello {
    my ($self, $fragment) = (shift, $_[0]);
    (my $major, my $minor, $self->{Pending}{Connection}{ServerRandom},
	$self->{Pending}{ID}, my $cipher_suite, $self->{Pending}{Compression}) =
	unpack "CC a32 C/a a2 C", $fragment;
    my $CipherSuite = $CipherSuite{$cipher_suite};
    $self->{Pending}{CipherSpec} = $CipherSpec{$CipherSuite};
    #print "ServerVersion=$major.$minor ServerTime=",scalar(localtime(unpack "N",$self->{Pending}{Connection}{ServerRandom})),"\n";
    #print "CipherSuite=$CipherSuite CompressionMethod=$CompressionMethod{$self->{Pending}{Compression}}\n";
    die "Bad server version" 
	if $major != $Protocol{major} or $minor != $Protocol{minor};
}

sub _ServerCertificate {
    my ($self, $fragment) = (shift, $_[0]);
    #    .
    my ($len1, $len) = unpack "Cn", $fragment;
    $len += $len1 << 16;
    $self->{Pending}{PeerCert} = substr $fragment, 3, $len;
    my $L   = qr/(?:\x82..|\x81.|.)/s;
    my $OID = qr/\x30$L\x30$L\x06\x09\x2A\x86\x48\x86\xF7\x0D\x01\x01\x01\x05\x00\x03$L\x00\x30$L/s;
    my $INT = qr/\x02($L)/s;
    die "_ServerCertificate: parse certificate" if $self->{Pending}{PeerCert} !~ /$OID$INT/gso;
    my $i1l = unpack((length($1)==3 ? "xn" : length($1)==2 ? "xC" : "C"), $1);
    $self->{Pending}{PeerCert} =~ /\G(.{$i1l})$INT/gs;
    $self->{Pending}{Connection}{ServerWriteKey}{m} = $1;
    my $i2l = unpack((length($2)==3 ? "xn" : length($2)==2 ? "xC" : "C"), $2);
    $self->{Pending}{PeerCert} =~ /\G(.{$i2l})/gs;
    $self->{Pending}{Connection}{ServerWriteKey}{e} = $1;
    #print "Modulus:\n".__DUMP($self->{Pending}{Connection}{ServerWriteKey}{m});
    #print "E:\n".__DUMP($self->{Pending}{Connection}{ServerWriteKey}{e})."\n";
}

sub _ServerCertificateRequest {
    my ($self, $fragment) = (shift, $_[0]);
    my @types = unpack "C/C*", $fragment;
    my ($names, @names) = unpack "x".(1+@types)."n/a*", $fragment;
    while ($names) {
        push @names, unpack("n/a*", $names);
	substr $names, 0, 2 + length($names[-1]), "";
    }
    $self->{cert_req} = 1;
}

sub _ServerKeyExchange {
    my ($self, $fragment) = (shift, $_[0]);
    if ($self->{Pending}{CipherSpec}{KeyExchange} eq "rsa") {
	my ($m, $e, $mac) = unpack "n/a n/a n/a", $fragment;
	my $hashdata = $self->{Pending}{Connection}{ClientRandom} .
	    $self->{Pending}{Connection}{ServerRandom} .
	    substr($fragment, 0, length($m)+length($e)+4);
	#print "Modulus:\n".__DUMP($m);
	#print "E:\n".__DUMP($e);
	my $md5sha1 = i2osp( RSA(os2ip($mac),
	    os2ip($self->{Pending}{Connection}{ServerWriteKey}{m}),
	    os2ip($self->{Pending}{Connection}{ServerWriteKey}{e})
	    ) );
	# FIXME padding: PKCS #1 block type 1
	die "_ServerKeyExchange: bad PKCSv1.5 padding"
	    unless $md5sha1 =~ s/^\x01\xFF*\x00//; # remove EMSA-PKCS1-v1_5 padding
	die "_ServerKeyExchange: bad mac" if $md5sha1 ne md5($hashdata).sha1($hashdata);
	@{$self->{Pending}{Connection}{ServerWriteKey}}{"m","e"} = ($m, $e);
    } else {
	die "_ServerKeyExchange: KeyExchange = $self->{Pending}{CipherSpec}{KeyExchange}?";
    }
}

sub _ServerHelloDone {
    my $self = shift;
    # FIXME TRACE
    #$self->_pack_alert('warning', 'no_certificate') if $self->{cert_req};
    if ($self->{cert_req}) {
        $self->{freeze_handshake} = 1;
	$self->_pack_handshake('certificate', __trace2bin("
   00 05 2b                                          
   00 02 7b                                          
   30 82 02 77 30 82 01 e0 a0 03 02 01 02 02 01 74   
   30 0d 06 09 2a 86 48 86 f7 0d 01 01 04 05 00 30   
   77 31 0b 30 09 06 03 55 04 06 13 02 55 53 31 2c   
   30 2a 06 03 55 04 0a 13 23 4e 65 74 73 63 61 70   
   65 20 43 6f 6d 6d 75 6e 69 63 61 74 69 6f 6e 73   
   20 43 6f 72 70 6f 72 61 74 69 6f 6e 31 11 30 0f   
   06 03 55 04 0b 13 08 48 61 72 64 63 6f 72 65 31   
   27 30 25 06 03 55 04 03 13 1e 48 61 72 64 63 6f   
   72 65 20 43 65 72 74 69 66 69 63 61 74 65 20 53   
   65 72 76 65 72 20 49 49 30 1e 17 0d 39 37 30 38   
   32 32 30 31 35 38 30 39 5a 17 0d 39 38 30 32 31   
   38 30 31 35 38 30 39 5a 30 81 97 31 0b 30 09 06   
   03 55 04 06 13 02 55 53 31 11 30 0f 06 03 55 04   
   0a 13 08 4e 65 74 73 63 61 70 65 31 1a 30 18 06   
   03 55 04 0b 13 11 48 61 72 64 63 6f 72 65 20 53   
   65 63 75 72 69 74 79 31 18 30 16 06 0a 09 92 26   
   89 93 f2 2c 64 01 01 13 08 4a 51 54 65 73 74 65   
   72 31 1a 30 18 06 03 55 04 03 13 11 4a 6f 68 6e   
   20 51 20 53 53 4c 20 54 65 73 74 65 72 31 23 30   
   21 06 09 2a 86 48 86 f7 0d 01 09 01 16 14 4a 51   
   54 65 73 74 65 72 40 6e 6f 77 68 65 72 65 2e 6f   
   72 67 30 5c 30 0d 06 09 2a 86 48 86 f7 0d 01 01   
   01 05 00 03 4b 00 30 48 02 41 00 e6 7a 1c b7 47   
   81 63 39 a9 0f 07 28 8a 7e e9 01 b1 bc c5 2a f0   
   ef ee bf 79 3f 11 9c 40 07 0a 09 69 fa a9 a9 c2   
   9f bc 05 aa 20 39 eb b5 23 38 07 e9 2a 67 0d 11   
   b8 9d 28 16 1d 62 b0 e2 4a 51 c1 02 03 01 00 01   
   a3 36 30 34 30 11 06 09 60 86 48 01 86 f8 42 01   
   01 04 04 03 02 00 80 30 1f 06 03 55 1d 23 04 18   
   30 16 80 14 97 b1 6d b2 b6 02 16 54 0c 97 d7 e3   
   32 6d cb 9c df ee de 80 30 0d 06 09 2a 86 48 86   
   f7 0d 01 01 04 05 00 03 81 81 00 22 44 a0 00 27   
   c0 ec c7 65 77 ec 55 af cd 31 93 e0 fb 2a c1 94   
   05 da 4a b3 f6 4c 15 5d 8d 52 c0 42 5e 8d f4 0d   
   6a 4e d0 8e ed f0 ff 78 2e ea 8f 0c ff 8e f6 9d   
   f9 ed b8 99 84 b5 4d 58 2b 81 ff c5 53 6a ed 5a   
   86 b6 67 13 62 e6 b4 07 77 4a ef 48 1a d3 3d ae   
   51 c5 7f 3e 02 f5 40 91 bd a8 d3 17 94 6b 49 86   
   38 95 74 4b e8 b7 1d 68 5a 3a c9 d0 b1 10 9f 90   
   e5 6b 0b c4 db 19 c7 b1 bb e2 8a                  
   00 02 aa                                          
   30 82 02 a6 30 82 02 0f a0 03 02 01 02 02 01 01   
   30 0d 06 09 2a 86 48 86 f7 0d 01 01 05 05 00 30   
   77 31 0b 30 09 06 03 55 04 06 13 02 55 53 31 2c   
   30 2a 06 03 55 04 0a 13 23 4e 65 74 73 63 61 70   
   65 20 43 6f 6d 6d 75 6e 69 63 61 74 69 6f 6e 73   
   20 43 6f 72 70 6f 72 61 74 69 6f 6e 31 11 30 0f   
   06 03 55 04 0b 13 08 48 61 72 64 63 6f 72 65 31   
   27 30 25 06 03 55 04 03 13 1e 48 61 72 64 63 6f   
   72 65 20 43 65 72 74 69 66 69 63 61 74 65 20 53   
   65 72 76 65 72 20 49 49 30 1e 17 0d 39 37 30 35   
   32 37 31 38 30 39 34 37 5a 17 0d 39 38 30 35 32   
   37 31 38 30 39 34 37 5a 30 77 31 0b 30 09 06 03   
   55 04 06 13 02 55 53 31 2c 30 2a 06 03 55 04 0a   
   13 23 4e 65 74 73 63 61 70 65 20 43 6f 6d 6d 75   
   6e 69 63 61 74 69 6f 6e 73 20 43 6f 72 70 6f 72   
   61 74 69 6f 6e 31 11 30 0f 06 03 55 04 0b 13 08   
   48 61 72 64 63 6f 72 65 31 27 30 25 06 03 55 04   
   03 13 1e 48 61 72 64 63 6f 72 65 20 43 65 72 74   
   69 66 69 63 61 74 65 20 53 65 72 76 65 72 20 49   
   49 30 81 9f 30 0d 06 09 2a 86 48 86 f7 0d 01 01   
   01 05 00 03 81 8d 00 30 81 89 02 81 81 00 bc 14   
   a0 c0 53 fa e1 4d b9 cd 0e b7 42 e3 cd 98 46 e0   
   b3 1e 13 76 c7 c5 e5 3d e5 24 18 dd 72 1a 37 7f   
   c4 66 51 36 7f e1 ae e9 11 5e 29 6f ac ff 28 ce   
   cd 53 ae 39 09 75 a1 eb d2 ec 79 d4 e9 6b 4c 99   
   e4 b6 42 d0 f7 52 8b ae 4a 33 6b 58 5b 47 57 13   
   a3 61 32 86 02 e8 63 e6 7a 27 c2 99 7a 22 48 d9   
   c8 d1 5c 6d b1 37 84 66 4b 9e a2 ce 31 6c 1c 06   
   7a 5f c5 7b b8 ff 58 89 f6 0b 40 6f 7c 0d 02 03   
   01 00 01 a3 42 30 40 30 1d 06 03 55 1d 0e 04 16   
   04 14 97 b1 6d b2 b6 02 16 54 0c 97 d7 e3 32 6d   
   cb 9c df ee de 80 30 1f 06 03 55 1d 23 04 18 30   
   16 80 14 97 b1 6d b2 b6 02 16 54 0c 97 d7 e3 32   
   6d cb 9c df ee de 80 30 0d 06 09 2a 86 48 86 f7   
   0d 01 01 05 05 00 03 81 81 00 9b 52 fe 93 fa 40   
   4d a9 8d 72 f9 f6 f6 c9 32 40 dc 20 fe be a5 a2   
   db e6 2c df d1 5f a0 66 45 d1 6e 5f 0a 91 e9 0b   
   c1 7c 8a c0 64 a0 d4 24 56 85 b5 a0 aa 1e c8 8c   
   15 40 ac fc 5a 2f 94 18 44 b9 73 23 c1 49 a0 24   
   ff b0 47 9c d8 28 1f b3 70 a7 62 b3 5b 8e 4d 82   
   bd 4d 85 eb 0d 5a 87 c0 41 c9 a6 c2 69 9c ee 81   
   49 2a fb 01 55 6f b1 df 21 a7 b0 70 e4 5d 34 3b   
   90 29 f9 14 c3 2e 07 79 13 c7                     
	    "));
        $self->_ClientKeyExchange();
        $self->{freeze_handshake} = 0;
	$self->_pack_handshake('certificate_verify', __trace2bin("
   00 40                                             
   40 86 2f b5 b7 d4 18 74 e8 89 07 a0 38 eb 2f ec   
   99 cc 10 5d 29 29 c2 08 61 21 aa 7d f7 64 c6 6a   
   62 77 28 96 8e b5 61 74 f2 84 1c 31 a2 3f 4f 58   
   78 11 88 72 4e ed 70 e8 ac e9 82 37 ec 8d 32 7b   
	    "));
    } else {
        $self->_ClientKeyExchange();
    }
    $self->_ClientChangeCipherSpec();
    $self->_ClientFinished();
    $self->{io} = "write";
}

sub _ClientKeyExchange {
    my $self = shift;
    if ($self->{Pending}{CipherSpec}{KeyExchange} eq "rsa") {
	open my $RND, "/dev/urandom" or die "open /dev/urandom: $!";
	die "can't read /dev/urandom" if 46 != sysread($RND, my $rnd46, 46);
	close $RND;
	# FIXME TRACE
	$rnd46 = __trace2bin("
         fc 4e 76 86 6b 74 7f 24 d8 12 07 78 bd 29   
   f6 b4 fa 34 7e d4 6a 8a 26 d7 2e 94 6c aa 1f 80   
   b9 75 b1 70 0f 7a 07 ce 8a db f7 98 56 f8 8b e6   
	    ");
	$self->{Pending}{PreMasterSecret} = $Protocol.$rnd46;
	#print "blocksize = ",(blocksize(encryptblock(
	#    os2ip($self->{Pending}{Connection}{ServerWriteKey}{m})),
	#    $self->{Pending}{PreMasterSecret})),"\n";
	my $PKCS = PKCS1v15_block2($self->{Pending}{PreMasterSecret},
	    os2ip($self->{Pending}{Connection}{ServerWriteKey}{m}));
	#print "PreMasterSecret:\n".__DUMP($self->{Pending}{PreMasterSecret});
	#print "PKCS:\n".__DUMP($PKCS);
	my $fragment = i2osp( RSA( os2ip($PKCS),
	    os2ip($self->{Pending}{Connection}{ServerWriteKey}{m}),
	    os2ip($self->{Pending}{Connection}{ServerWriteKey}{e}) ));
	$self->_pack_handshake("client_key_exchange", $fragment);
    } else {
	die "_ClientKeyExchange: KeyExchange = $self->{Pending}{CipherSpec}{KeyExchange}?";
    }
}

sub _ClientChangeCipherSpec {
    my $self = shift;
    $self->_pack("change_cipher_spec", "\x01");
    $self->{Pending}{MasterSecret} = join "",
	map { md5($self->{Pending}{PreMasterSecret} .
	    sha1($_ . $self->{Pending}{PreMasterSecret} . 
		$self->{Pending}{Connection}{ClientRandom} .
		$self->{Pending}{Connection}{ServerRandom})) } qw(A BB CCC);
    delete $self->{Pending}{PreMasterSecret};
    my $key_block = "";
    # FIXME ..    RC4,  {IV_size}  
    #print "hash_size = $self->{Pending}{CipherSpec}{hash_size}\n";
    #print "key_material = $self->{Pending}{CipherSpec}{key_material}\n";
    my $key_block_size = $self->{Pending}{CipherSpec}{hash_size} * 2 +
	$self->{Pending}{CipherSpec}{key_material} * 2;
    my $s = "A";
    while (length($key_block) < $key_block_size) {
	#print "KEY_BLOCK:\n".__DUMP($s);
	$key_block .= md5($self->{Pending}{MasterSecret} .
	    sha1($s . $self->{Pending}{MasterSecret} .
		$self->{Pending}{Connection}{ServerRandom} .
		$self->{Pending}{Connection}{ClientRandom}));
	$s = substr(++$s, -1) x (length($s)+1); # A -> BB -> CCC -> ...
    }
    @{$self->{Pending}{Connection}}{"ClientWriteMACSecret",
	"ServerWriteMACSecret","ClientWriteKey","ServerWriteKey"} =
	unpack(("a$self->{Pending}{CipherSpec}{hash_size}"x2).
	("a$self->{Pending}{CipherSpec}{key_material}"x2), $key_block);
    #print "ClientWriteMACSecret:\n".__DUMP($self->{Pending}{Connection}{ClientWriteMACSecret});
    if ($self->{Pending}{CipherSpec}{is_exportable} eq "true") {
	$self->{Pending}{Connection}{ClientWriteKey} = md5(
	    $self->{Pending}{Connection}{ClientWriteKey} .
	    $self->{Pending}{Connection}{ClientRandom} .
	    $self->{Pending}{Connection}{ServerRandom});
	$self->{Pending}{Connection}{ServerWriteKey} = md5(
	    $self->{Pending}{Connection}{ServerWriteKey} .
	    $self->{Pending}{Connection}{ServerRandom} .
	    $self->{Pending}{Connection}{ClientRandom});
    }
    $self->{Pending}{Connection}{ClientWriteRC4} =
	Crypt::RC4->new($self->{Pending}{Connection}{ClientWriteKey});
    $self->{Pending}{Connection}{ServerWriteRC4} =
	Crypt::RC4->new($self->{Pending}{Connection}{ServerWriteKey});
    $self->{Pending}{Connection}{ClientSEQ} = 0;
    $self->{Write} = $self->{Pending};
}

sub _ClientFinished {
    my $self = shift;
    my $fragment =
	md5($self->{Pending}{MasterSecret} . $pad2{md5} .
	    md5($self->{Pending}{HandshakeMessages} . $Finished{client} .
	    $self->{Pending}{MasterSecret} . $pad1{md5})) .
	sha1($self->{Pending}{MasterSecret} . $pad2{sha} .
	    sha1($self->{Pending}{HandshakeMessages} . $Finished{client} .
	    $self->{Pending}{MasterSecret} . $pad1{sha}));
    $self->_pack_handshake("finished", $fragment);
    $self->{finished} |= 0b01;
}

sub _ServerChangeCipherSpec {
    my $self = shift;
    $self->{Pending}{Connection}{ServerSEQ} = 0;
    $self->{Read} = $self->{Pending};
}

sub _ServerFinished {
    my ($self, $fragment) = (shift, $_[0]);
    my $finished = 
	md5($self->{Pending}{MasterSecret} . $pad2{md5} .
	    md5($self->{Pending}{HandshakeMessages} . $Finished{server} .
	    $self->{Pending}{MasterSecret} . $pad1{md5})) .
	sha1($self->{Pending}{MasterSecret} . $pad2{sha} .
	    sha1($self->{Pending}{HandshakeMessages} . $Finished{server} .
	    $self->{Pending}{MasterSecret} . $pad1{sha}));
    die "_ServerFinished: bad finished message" if $fragment ne $finished;
    $self->{finished} |= 0b10;
}

sub _ServerAlert_CloseNofity {
    my $self = shift;
    $self->{closed} = 1;
}

sub _pack_alert {
    my ($self, $level, $description) = (shift, @_);
    $self->_pack('alert', pack("CC", $AlertLevel{$level},
	$AlertDescription{$description}));
}

sub _pack_handshake {
    my ($self, $Type, $fragment) = (shift, $_[0], $_[1]);
warn " >   $Type\n";
    #print " >   $Type\n";
    my $length = length($fragment);
    $fragment = pack("C Cn a*", $HandshakeType{$Type},
	($length & 0x00FF0000) >> 16, $length & 0x0000FFFF, $fragment);
    $self->{Pending}{HandshakeMessages} .= $fragment;
    # FIXME TRACE
    if ($self->{freeze_handshake}) {
	$self->{freeze_handshake_fragment} .= $fragment;
	return;
    } else {
	$fragment = $self->{freeze_handshake_fragment} . $fragment;
	$self->{freeze_handshake_fragment} = "";
    }
    $self->_pack("handshake", $fragment);
}

sub _unpack_handshake {
    my ($self, $fragment) = (shift, $_[0]);
    # FIXME   hello_request,    ?
    $self->{Pending}{HandshakeMessages} .= $fragment;
    my $handshake_message = $fragment;
##    (my $Type, my $len1, my $len, $fragment) = unpack "C Cn a*", $fragment;
##    $len += $len1 << 16;

  while (length $fragment) {
    my ($Type, $len1, $len) = unpack "C Cn", $fragment;
    $len += $len1 << 16;
##    my $handshake_message = substr($fragment, 0, 4+$len, "");
##    $self->{Pending}{HandshakeMessages} .= $handshake_message;
##    my $fragment = substr($handshake_message, 4);
    my $fragment = substr($fragment, 0, 4+$len, "");
    substr($fragment, 0, 4, "");
      
warn " <   $HandshakeType{$Type}\n";
    #print " <   $HandshakeType{$Type}\n";
die "_unpack_handshake: type='$HandshakeType{$Type}' bad fragment len ".
 length($fragment)." (should be $len)\n".__DUMP($fragment)
 unless $len == length($fragment);
    die "_unpack_handshake: bad fragment" unless $len == length($fragment);
    if ($Type == $HandshakeType{server_hello}) {
	$self->_ServerHello($fragment);
    } elsif ($Type == $HandshakeType{certificate}) {
	$self->_ServerCertificate($fragment);
    } elsif ($Type == $HandshakeType{certificate_request}) {
	$self->_ServerCertificateRequest($fragment);
    } elsif ($Type == $HandshakeType{hello_request}) {
	die "_unpack_handshake: hello_request...";
    } elsif ($Type == $HandshakeType{server_key_exchange}) {
	$self->_ServerKeyExchange($fragment);
    } elsif ($Type == $HandshakeType{server_hello_done}) {
	$self->_ServerHelloDone();
    } elsif ($Type == $HandshakeType{finished}) {
	$self->{Pending}{HandshakeMessages} =~ s/\Q$handshake_message\E$//;
	$self->_ServerFinished($fragment);
	$self->{Pending}{HandshakeMessages} .= $handshake_message;
    } else {
	die "_unpack_handshake: type = $Type?";
    }
  }
}

sub _pack {
    my ($self, $Type, $fragment) = @_;
warn " --> $Type\n"; 
    #print " --> $Type\n"; 
    ### SSLPlaintext
    my $type	= pack("C", $ContentType{$Type});
    my $length	= length($fragment);
    ### Fragmentation
    die "_pack: fragment too big" if $length > $MaxFragmentLength;
    ### SSLCompressed
    if ($self->{Write}{Compression} != $CompressionMethod{null}) {
	die "_pack: Compression = $self->{Write}{Compression}?";
    }
    ### SSLCiphertext
#    print "ClientWriteMACSecret:\n".__DUMP($self->{Write}{Connection}{ClientWriteMACSecret});
#    print "pad1:\n".__DUMP($pad1{md5});
#    print "seq:\n".__DUMP(pack("NN", 0, $self->{Write}{Connection}{ClientSEQ}));
#    print "length:\n".__DUMP(pack("n", $length));
#    print "fragment:\n".__DUMP($fragment);
#    print "MD5 internal:\n".__DUMP(
#	md5($self->{Write}{Connection}{ClientWriteMACSecret},
#	$pad1{md5}, pack("NN", 0, $self->{Write}{Connection}{ClientSEQ}),
#	"\x16", pack("n", $length), $fragment));
    if ($self->{Write}{CipherSpec}{mac_algorithm} eq "null") {
    } elsif ($self->{Write}{CipherSpec}{mac_algorithm} eq "md5") {
	$fragment .= md5($self->{Write}{Connection}{ClientWriteMACSecret},
	    $pad2{md5}, md5($self->{Write}{Connection}{ClientWriteMACSecret},
	    $pad1{md5}, pack("NN", 0, $self->{Write}{Connection}{ClientSEQ}),
	    $type, pack("n", $length), $fragment));
    } else {
	die "_pack: mac_algorithm = $self->{Write}{CipherSpec}{mac_algorithm}?";
    }
    $self->{Write}{Connection}{ClientSEQ}++;
    #print "Send packet header:\n".__DUMP($type . $Protocol . pack("n", $length));
    #print "Send packet content:\n".__DUMP($fragment);
    if ($self->{Write}{CipherSpec}{bulk_cipher_algorithm} eq "null") {
    } elsif ($self->{Write}{CipherSpec}{bulk_cipher_algorithm} eq "rc4") {
	#print "ClientWriteKey:\n".__DUMP($self->{Write}{Connection}{ClientWriteKey});
        $fragment = $self->{Write}{Connection}{ClientWriteRC4}->RC4($fragment);
    } else {
	die "_pack: bulk_cipher_algorithm = $self->{Write}{CipherSpec}{bulk_cipher_algorithm}?";
    }
    #print "Send packet content (encrypted):\n".__DUMP($fragment);
    $length	= $length + $self->{Write}{CipherSpec}{hash_size};
    $self->{writebuf} .= $type . $Protocol . pack("n", $length) . $fragment;
warn __DUMP($type . $Protocol . pack("n", $length) . $fragment);
}

sub _unpack {
    my $self = shift;
    return if length($self->{readbuf}) < 5;
    my ($type, $major, $minor, $length) = unpack("CCCn", $self->{readbuf});
    return if length($self->{readbuf}) < 5 + $length;
    die "_unpack: Protocol = $major.$minor?"
	if $major != $Protocol{major} or $minor != $Protocol{minor};
    ### SSLCiphertext
    my $fragment = substr($self->{readbuf}, 5, $length);
my $DUMP = __DUMP(substr($self->{readbuf}, 0, 5 + $length));
    $self->{readbuf} = substr($self->{readbuf}, 5 + $length);
    if ($self->{Read}{CipherSpec}{bulk_cipher_algorithm} eq "null") {
    } elsif ($self->{Read}{CipherSpec}{bulk_cipher_algorithm} eq "rc4") {
	#$fragment = RC4($self->{Read}{Connection}{ServerWriteKey}, $fragment);
        $fragment = $self->{Read}{Connection}{ServerWriteRC4}->RC4($fragment);
    } else {
	die "_unpack: bulk_cipher_algorithm = $self->{Read}{CipherSpec}{bulk_cipher_algorithm}?";
    }
    my $mac = substr($fragment, $length-=$self->{Read}{CipherSpec}{hash_size},
	$self->{Read}{CipherSpec}{hash_size}, "");
    if ($self->{Read}{CipherSpec}{mac_algorithm} eq "null") {
    } elsif ($self->{Read}{CipherSpec}{mac_algorithm} eq "md5") {
	#print "type = $type\n";
	#print "fragment:\n".__DUMP($fragment);
	#print "mymac:\n".__DUMP($mac);
	#print "mac:\n".__DUMP(
	#    md5($self->{Read}{Connection}{ServerWriteMACSecret}, $pad2{md5},
	#    md5($self->{Read}{Connection}{ServerWriteMACSecret}, $pad1{md5},
	#    pack("NN", 0, $self->{Read}{Connection}{ServerSEQ}),
	#    pack("C", $type), pack("n", $length), $fragment)));
	die "_unpack: bad mac" unless $mac eq 
	    md5($self->{Read}{Connection}{ServerWriteMACSecret}, $pad2{md5},
	    md5($self->{Read}{Connection}{ServerWriteMACSecret}, $pad1{md5},
	    pack("NN", 0, $self->{Read}{Connection}{ServerSEQ}),
	    pack("C", $type), pack("n", $length), $fragment));
    } else {
	die "_unpack: mac_algorithm = $self->{Read}{CipherSpec}{mac_algorithm}?";
    }
    $self->{Read}{Connection}{ServerSEQ}++;
    ### SSLCompressed
    if ($self->{Read}{Compression} != $CompressionMethod{null}) {
	die "_unpack: Compression = $self->{Read}{Compression}?";
    }
    ### Fragmentation
    die "_unpack: fragment too big" if $length > $MaxFragmentLength;
    ### SSLPlaintext
warn " <-- $ContentType{$type}\n";
warn $DUMP;
    #print " <-- $ContentType{$type}\n";
    if ($type == $ContentType{application_data}) {
	$self->{readed} .= $fragment;
    } elsif ($type == $ContentType{change_cipher_spec}) {
	die "_unpack: bad fragment" if $length != 1 or
	    $ChangeCipherSpec{change_cipher_spec} != unpack("C", $fragment);
	$self->_ServerChangeCipherSpec();
    } elsif ($type == $ContentType{alert}) {
	die "_unpack: bad fragment" if $length != 2;
	my ($level, $description) = unpack "CC", $fragment;
	if ($AlertLevel{$level} eq "warning" and
		$AlertDescription{$description} eq "close_notify") {
	    $self->_ServerAlert_CloseNofity();
	} else {
	    die "_unpack: AlertLevel=$AlertLevel{$level}, AlertDescription=$AlertDescription{$description}\n";
	}
    } elsif ($type == $ContentType{handshake}) {
	$self->_unpack_handshake($fragment);
    } else {
	die "_unpack: unknown type = $type?";
    }
}

sub _Handshake_io {
    my $self = shift;
    #print "_Handshake_io ==> io=$self->{io}\n";
    #print "_Handshake_io ==> readbuf=",length($self->{readbuf}),"\n";
    $! = 0 if $! == EAGAIN;
    return 0 if $self->{closed};
    $self->_unpack() if $self->{readbuf};
    if ($self->{io} eq "read") {
	my $n = sysread($self->{SOCK}, my $buf, 100000);
	return if not defined $n;
	return 0 if defined $n and $n == 0 and $! == 0;
	$self->{readbuf} .= substr($buf,0,$n);
    } else {
	return if not defined(my $n = syswrite($self->{SOCK}, $self->{writebuf}));
	$self->{io} = "read" unless $self->{writebuf} = substr($self->{writebuf}, $n);
    }
    $! ||= EAGAIN; return undef;
}

sub __DUMP {
    my $dump = join " ", map {$_=sprintf("%02X",$_)} unpack("C*", join("", @_));
    $dump =~ s/(\w\w \w\w \w\w \w\w) /$1    /g;
    $dump =~ s/(.*?    .*?    .*?    .*?)    /$1\n/g;
    return $dump."\n";
}

# FIXME TRACE
sub __trace2bin {
    (my $trace = uc($_[0])) =~ s/\s+//g;
    die "trace2bin" if length($trace) % 2 or $trace =~ /[^0-9A-F]/;
    $trace =~ s/(..)/chr(hex($1))/eg;
    return $trace;
}

=item B<new>( $socket )



=cut

sub new {
    my $this  = shift;
    my $class = ref($this) || $this;
    my $self  = {
	SOCK	    => $_[0],
	io	    => "write",	    #  writebuf    readbuf
	closed	    => 0,
	finished    => 0,	    # |= 0b01   ClientFinished
				    # |= 0b10   ServerFinished
	writebuf    => "",
	readbuf	    => "",
	readed	    => "",	    # readed application_data, decrypted
	cert_req    => 0,	    # is server request client's certificate?
	Read	    => _get_clean_session(),
	Write	    => _get_clean_session(),
	Pending	    => _get_clean_session(),
	};
    bless $self, $class;
    $self->_ClientHello();
    return $self;
}

=item B<sslread>( )



=cut

sub sslread {
    my ($self, $buf, $max) = (shift, "", $_[1]);
    if ($self->{finished} != 0b11 or not $self->{readed}) {
	my $n = $self->_Handshake_io();
	return $n if not $self->{readed};
    }
#    return if not defined(my $n = sysread($self->{SOCK}, $buf, $max));
#    return 0 unless $n;
#    $self->{readbuf} .= substr($buf,0,$n);
#    $self->_unpack();
    $_[0] = $self->{readed};
    $self->{readed} = "";
    $!=0;
    return length($_[0]);
}

=item B<sslwrite>( $data )



=cut

sub sslwrite {
    my ($self, $data) = (shift, $_[0]);
    return $self->_Handshake_io() if $self->{finished} != 0b11 or $self->{writebuf};
    $self->_pack("application_data", $data);
    $self->{io} = "write";
    $!=0;
    return length($data);
}

=back

=head1 VARIABLES

=over 4

=item B<>



=back

=head1 TODO

 SessionCache:
 -  SessionCache  ,   
 -   new()  _ClientHello()    
   SessionID,   IP   
    {SOCK}
 - {Session}{IsResumable} = "true" ?

 BlockingSockets:
 -    sslread()  sslwrite()   ,
      Handshake  /    
 -   EAGAIN  _Handshake_io()
 -   5    $length ,    

 ClientCapabilities:
 -  $CipherSuites  $CompressionMethods
 -   ?

 Version:
 -  SSL2  TLS?

 Fragmentation:
 -  /   $MaxFragmentLength?

 RSA:
 -  POWER::RSA   encrypt/decrypt/sign/verify,
     PKCS1v15_block1  sign/verify, PKCS1v15_block2 
   blocksize  encrypt/decrypt.
 -   i2osp()  os2ip()
 -   POWER::RSA    / 
   (  openssl -   Crypt::RSA  )

 Errors/Misc:
 -      alert  
 -      
 -   alert
 -        authorities
 -  {Pending}  {Session}  {read_pending}, {read_current},
   {write_pending}, {write_current}
 - ,     RSA ,   
    os2ip/i2osp/octet_len
 -    {ServerWriteKey}  RSA'  {m=>$m,e=>$e},  
   RC4'  $key... !
 -   Makefile.PL   
 -  ?
 -      alert   ssl 

=head1 HISTORY

 0.02		   8 22:41:58 EET 2003
      - bugfix:    handshake  
	 ssl 
      - bugfix:  PKCS1v15_block1  PKCS1v15_block2   "\00"
          
      - bugfix:   ClientWriteKey   cipher
         
 0.01		   8 16:48:10 EET 2003
      - initial release

=head1 AUTHOR

Alex Efros <powerman@asdfGroup.com>

=cut

